@@ -51,9 +51,7 @@ module Language.Haskell.TH.Desugar (
   DerivingClause, dsDerivClause, dsLetDec,
   dsMatches, dsBody, dsGuards, dsDoStmts, dsComp, dsClauses,
   dsBangType, dsVarBangType,
-#if __GLASGOW_HASKELL__ > 710
   dsTypeFamilyHead, dsFamilyResultSig,
-#endif
 #if __GLASGOW_HASKELL__ >= 801
   dsPatSynDir,
 #endif
@@ -97,10 +95,7 @@ module Language.Haskell.TH.Desugar (
   tupleDegree_maybe, tupleNameDegree_maybe,
   unboxedSumDegree_maybe, unboxedSumNameDegree_maybe,
   unboxedTupleDegree_maybe, unboxedTupleNameDegree_maybe,
-  strictToBang, isTypeKindName, typeKindName,
-#if __GLASGOW_HASKELL__ >= 800
-  bindIP,
-#endif
+  isTypeKindName, typeKindName, bindIP,
   mkExtraDKindBinders, dTyVarBndrToDType, changeDTVFlags, toposortTyVarsOf,
 
   -- ** 'FunArgs' and 'VisFunArg'
@@ -136,15 +131,10 @@ import Language.Haskell.TH.Syntax
 import Control.Monad
 import qualified Data.Foldable as F
 import Data.Function
-import Data.List
 import qualified Data.Map as M
 import qualified Data.Set as S
 import Prelude hiding ( exp )
 
-#if __GLASGOW_HASKELL__ < 710
-import Control.Applicative
-#endif
-
 -- | This class relates a TH type with its th-desugar type and allows
 -- conversions back and forth. The functional dependency goes only one
 -- way because we define the following instances on old versions of GHC:
@@ -240,7 +230,7 @@ flattenDValD (DValD pat exp) = do
         DVarP n
           | n == name -> DVarP y
           | otherwise -> DWildP
-        DConP con ps -> DConP con (map (wildify name y) ps)
+        DConP con ts ps -> DConP con ts (map (wildify name y) ps)
         DTildeP pa -> DTildeP (wildify name y pa)
         DBangP pa -> DBangP (wildify name y pa)
         DSigP pa ty -> DSigP (wildify name y pa) ty
@@ -297,7 +287,7 @@ getRecordSelectors cons = merge_let_decs `fmap` concatMapM get_record_sels cons
             return $ concat
               [ [ DSigD name $ DForallT (DForallInvis con_tvbs)
                              $ DArrowT `DAppT` con_ret_ty `DAppT` field_ty
-                , DFunD name [DClause [DConP con_name
+                , DFunD name [DClause [DConP con_name []
                                          (mk_field_pats n (length fields) varName)]
                                       (DVarE varName)] ]
               | ((name, _strict, field_ty), n) <- zip fields [0..]
