@@ -11,9 +11,6 @@ module Language.Haskell.TH.Desugar.FV
   , extractBoundNamesDPat
   ) where
 
-#if __GLASGOW_HASKELL__ < 710
-import Data.Foldable (foldMap)
-#endif
 #if __GLASGOW_HASKELL__ < 804
 import Data.Monoid ((<>))
 #endif
@@ -49,13 +46,13 @@ extractBoundNamesDPat :: DPat -> OSet Name
 extractBoundNamesDPat = go
   where
     go :: DPat -> OSet Name
-    go (DLitP _)      = OS.empty
-    go (DVarP n)      = OS.singleton n
-    go (DConP _ pats) = foldMap go pats
-    go (DTildeP p)    = go p
-    go (DBangP p)     = go p
-    go (DSigP p _)    = go p
-    go DWildP         = OS.empty
+    go (DLitP _)          = OS.empty
+    go (DVarP n)          = OS.singleton n
+    go (DConP _ tys pats) = foldMap fvDType tys <> foldMap go pats
+    go (DTildeP p)        = go p
+    go (DBangP p)         = go p
+    go (DSigP p _)        = go p
+    go DWildP             = OS.empty
 
 -----
 -- Binding forms
