@@ -7,7 +7,7 @@ Converts desugared TH back into real TH.
 -}
 
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskellQuotes #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -43,7 +43,7 @@ import Language.Haskell.TH hiding (cxt)
 import Language.Haskell.TH.Datatype.TyVarBndr
 
 import Language.Haskell.TH.Desugar.AST
-import Language.Haskell.TH.Desugar.Core (DTypeArg(..), changeDTVFlags)
+import Language.Haskell.TH.Desugar.Core (DTypeArg(..))
 import Language.Haskell.TH.Desugar.Util
 
 expToTH :: DExp -> Exp
@@ -55,11 +55,7 @@ expToTH (DLamE names exp)    = LamE (map VarP names) (expToTH exp)
 expToTH (DCaseE exp matches) = CaseE (expToTH exp) (map matchToTH matches)
 expToTH (DLetE decs exp)     = LetE (map letDecToTH decs) (expToTH exp)
 expToTH (DSigE exp ty)       = SigE (expToTH exp) (typeToTH ty)
-#if __GLASGOW_HASKELL__ < 709
-expToTH (DStaticE _)         = error "Static expressions supported only in GHC 7.10+"
-#else
 expToTH (DStaticE exp)       = StaticE (expToTH exp)
-#endif
 #if __GLASGOW_HASKELL__ >= 801
 expToTH (DAppTypeE exp ty)   = AppTypeE (expToTH exp) (typeToTH ty)
 #else
@@ -72,13 +68,17 @@ matchToTH :: DMatch -> Match
 matchToTH (DMatch pat exp) = Match (patToTH pat) (NormalB (expToTH exp)) []
 
 patToTH :: DPat -> Pat
-patToTH (DLitP lit)    = LitP lit
-patToTH (DVarP n)      = VarP n
-patToTH (DConP n pats) = ConP n (map patToTH pats)
-patToTH (DTildeP pat)  = TildeP (patToTH pat)
-patToTH (DBangP pat)   = BangP (patToTH pat)
-patToTH (DSigP pat ty) = SigP (patToTH pat) (typeToTH ty)
-patToTH DWildP         = WildP
+patToTH (DLitP lit)         = LitP lit
+patToTH (DVarP n)           = VarP n
+patToTH (DConP n _tys pats) = ConP n
+#if __GLASGOW_HASKELL__ >= 901
+                                   (map typeToTH _tys)
+#endif
+                                   (map patToTH pats)
+patToTH (DTildeP pat)       = TildeP (patToTH pat)
+patToTH (DBangP pat)        = BangP (patToTH pat)
+patToTH (DSigP pat ty)      = SigP (patToTH pat) (typeToTH ty)
+patToTH DWildP              = WildP
 
 decsToTH :: [DDec] -> [Dec]
 decsToTH = map decToTH
@@ -88,53 +88,24 @@ decsToTH = map decToTH
 decToTH :: DDec -> Dec
 decToTH (DLetDec d) = letDecToTH d
 decToTH (DDataD Data cxt n tvbs _mk cons derivings) =
-#if __GLASGOW_HASKELL__ > 710
   DataD (cxtToTH cxt) n (map tvbToTH tvbs) (fmap typeToTH _mk) (map conToTH cons)
         (concatMap derivClauseToTH derivings)
-#else
-  DataD (cxtToTH cxt) n (map tvbToTH tvbs) (map conToTH cons)
-        (map derivingToTH derivings)
-#endif
 decToTH (DDataD Newtype cxt n tvbs _mk [con] derivings) =
-#if __GLASGOW_HASKELL__ > 710
   NewtypeD (cxtToTH cxt) n (map tvbToTH tvbs) (fmap typeToTH _mk) (conToTH con)
            (concatMap derivClauseToTH derivings)
-#else
-  NewtypeD (cxtToTH cxt) n (map tvbToTH tvbs) (conToTH con)
-           (map derivingToTH derivings)
-#endif
 decToTH (DDataD Newtype _cxt _n _tvbs _mk _cons _derivings) =
   error "Newtype declaration without exactly 1 constructor."
 decToTH (DTySynD n tvbs ty) = TySynD n (map tvbToTH tvbs) (typeToTH ty)
 decToTH (DClassD cxt n tvbs fds decs) =
   ClassD (cxtToTH cxt) n (map tvbToTH tvbs) fds (decsToTH decs)
-decToTH (DInstanceD over mtvbs _cxt _ty decs) =
-  instanceDToTH over cxt' ty' decs
-  where
-  (cxt', ty') = case fmap (changeDTVFlags SpecifiedSpec) mtvbs of
-                  Nothing    -> (_cxt, _ty)
-                  Just _tvbs ->
-#if __GLASGOW_HASKELL__ < 800 || __GLASGOW_HASKELL__ >= 802
-                                ([], DForallT (DForallInvis _tvbs) $ DConstrainedT _cxt _ty)
-#else
-                                -- See #117
-                                error $ "Explicit foralls in instance declarations "
-                                     ++ "are broken on GHC 8.0."
-#endif
+decToTH (DInstanceD over _mtvbs cxt ty decs) =
+  -- We deliberately avoid sweetening _mtvbs. See #151.
+  instanceDToTH over cxt ty decs
 decToTH (DForeignD f) = ForeignD (foreignToTH f)
-#if __GLASGOW_HASKELL__ > 710
 decToTH (DOpenTypeFamilyD (DTypeFamilyHead n tvbs frs ann)) =
   OpenTypeFamilyD (TypeFamilyHead n (map tvbToTH tvbs) (frsToTH frs) ann)
-#else
-decToTH (DOpenTypeFamilyD (DTypeFamilyHead n tvbs frs _ann)) =
-  FamilyD TypeFam n (map tvbToTH tvbs) (frsToTH frs)
-#endif
 decToTH (DDataFamilyD n tvbs mk) =
-#if __GLASGOW_HASKELL__ > 710
   DataFamilyD n (map tvbToTH tvbs) (fmap typeToTH mk)
-#else
-  FamilyD DataFam n (map tvbToTH tvbs) (fmap typeToTH mk)
-#endif
 decToTH (DDataInstD nd cxt mtvbs lhs mk cons derivings) =
   let ndc = case (nd, cons) of
               (Newtype, [con]) -> DNewtypeCon con
@@ -148,34 +119,14 @@ decToTH (DTySynInstD eqn) =
   let (n, eqn') = tySynEqnToTH eqn in
   TySynInstD n eqn'
 #endif
-#if __GLASGOW_HASKELL__ > 710
 decToTH (DClosedTypeFamilyD (DTypeFamilyHead n tvbs frs ann) eqns) =
   ClosedTypeFamilyD (TypeFamilyHead n (map tvbToTH tvbs) (frsToTH frs) ann)
                     (map (snd . tySynEqnToTH) eqns)
-#else
-decToTH (DClosedTypeFamilyD (DTypeFamilyHead n tvbs frs _ann) eqns) =
-  ClosedTypeFamilyD n (map tvbToTH tvbs) (frsToTH frs) (map (snd . tySynEqnToTH) eqns)
-#endif
 decToTH (DRoleAnnotD n roles) = RoleAnnotD n roles
-decToTH (DStandaloneDerivD mds mtvbs _cxt _ty) =
-  standaloneDerivDToTH mds cxt' ty'
-  where
-  (cxt', ty') = case fmap (changeDTVFlags SpecifiedSpec) mtvbs of
-                  Nothing    -> (_cxt, _ty)
-                  Just _tvbs ->
-#if __GLASGOW_HASKELL__ < 710 || __GLASGOW_HASKELL__ >= 802
-                                ([], DForallT (DForallInvis _tvbs) $ DConstrainedT _cxt _ty)
-#else
-                                -- See #117
-                                error $ "Explicit foralls in standalone deriving declarations "
-                                     ++ "are broken on GHC 7.10 and 8.0."
-#endif
-#if __GLASGOW_HASKELL__ < 709
-decToTH (DDefaultSigD {})      =
-  error "Default method signatures supported only in GHC 7.10+"
-#else
+decToTH (DStandaloneDerivD mds _mtvbs cxt ty) =
+  -- We deliberately avoid sweetening _mtvbs. See #151.
+  standaloneDerivDToTH mds cxt ty
 decToTH (DDefaultSigD n ty)        = DefaultSigD n (typeToTH ty)
-#endif
 #if __GLASGOW_HASKELL__ >= 801
 decToTH (DPatSynD n args dir pat) = PatSynD n args (patSynDirToTH dir) (patToTH pat)
 decToTH (DPatSynSigD n ty)        = PatSynSigD n (typeToTH ty)
@@ -189,6 +140,12 @@ decToTH (DKiSigD n ki) = KiSigD n (typeToTH ki)
 decToTH (DKiSigD {})   =
   error "Standalone kind signatures supported only in GHC 8.10+"
 #endif
+#if __GLASGOW_HASKELL__ >= 903
+decToTH (DDefaultD tys) = DefaultD (map typeToTH tys)
+#else
+decToTH (DDefaultD{})   =
+  error "Default declarations supported only in GHC 9.4+"
+#endif
 
 #if __GLASGOW_HASKELL__ < 801
 patSynErr :: a
@@ -211,12 +168,9 @@ dataInstDecToTH ndc cxt _mtvbs lhs _mk derivings =
       NewtypeInstD (cxtToTH cxt) (fmap (fmap tvbToTH) _mtvbs) (typeToTH lhs)
                    (fmap typeToTH _mk) (conToTH con)
                    (concatMap derivClauseToTH derivings)
-#elif __GLASGOW_HASKELL__ > 710
+#else
       NewtypeInstD (cxtToTH cxt) _n _lhs_args (fmap typeToTH _mk) (conToTH con)
                    (concatMap derivClauseToTH derivings)
-#else
-      NewtypeInstD (cxtToTH cxt) _n _lhs_args (conToTH con)
-                   (map derivingToTH derivings)
 #endif
 
     DDataCons cons ->
@@ -224,12 +178,9 @@ dataInstDecToTH ndc cxt _mtvbs lhs _mk derivings =
       DataInstD (cxtToTH cxt) (fmap (fmap tvbToTH) _mtvbs) (typeToTH lhs)
                 (fmap typeToTH _mk) (map conToTH cons)
                 (concatMap derivClauseToTH derivings)
-#elif __GLASGOW_HASKELL__ > 710
+#else
       DataInstD (cxtToTH cxt) _n _lhs_args (fmap typeToTH _mk) (map conToTH cons)
                 (concatMap derivClauseToTH derivings)
-#else
-      DataInstD (cxtToTH cxt) _n _lhs_args (map conToTH cons)
-                (map derivingToTH derivings)
 #endif
   where
     _lhs' = typeToTH lhs
@@ -238,25 +189,10 @@ dataInstDecToTH ndc cxt _mtvbs lhs _mk derivings =
         (ConT n, lhs_args) -> (n, filterTANormals lhs_args)
         (_, _) -> error $ "Illegal data instance LHS: " ++ pprint _lhs'
 
-#if __GLASGOW_HASKELL__ > 710
 frsToTH :: DFamilyResultSig -> FamilyResultSig
 frsToTH DNoSig          = NoSig
 frsToTH (DKindSig k)    = KindSig (typeToTH k)
 frsToTH (DTyVarSig tvb) = TyVarSig (tvbToTH tvb)
-#else
-frsToTH :: DFamilyResultSig -> Maybe Kind
-frsToTH DNoSig                        = Nothing
-frsToTH (DKindSig k)                  = Just (typeToTH k)
-frsToTH (DTyVarSig (DPlainTV _ _))    = Nothing
-frsToTH (DTyVarSig (DKindedTV _ _ k)) = Just (typeToTH k)
-#endif
-
-#if __GLASGOW_HASKELL__ <= 710
-derivingToTH :: DDerivClause -> Name
-derivingToTH (DDerivClause _ [DConT nm]) = nm
-derivingToTH p =
-  error ("Template Haskell in GHC < 8.0 only allows simple derivings: " ++ show p)
-#endif
 
 -- | Sweeten a 'DLetDec'.
 letDecToTH :: DLetDec -> Dec
@@ -267,93 +203,27 @@ letDecToTH (DInfixD f name)     = InfixD f name
 letDecToTH (DPragmaD prag)      = PragmaD (pragmaToTH prag)
 
 conToTH :: DCon -> Con
-#if __GLASGOW_HASKELL__ > 710
 conToTH (DCon [] [] n (DNormalC _ stys) rty) =
   GadtC [n] (map (second typeToTH) stys) (typeToTH rty)
 conToTH (DCon [] [] n (DRecC vstys) rty) =
   RecGadtC [n] (map (thirdOf3 typeToTH) vstys) (typeToTH rty)
-#else
-conToTH (DCon [] [] n (DNormalC True [sty1, sty2]) _) =
-  InfixC ((bangToStrict *** typeToTH) sty1) n ((bangToStrict *** typeToTH) sty2)
--- Note: it's possible that someone could pass in a DNormalC value that
--- erroneously claims that it's declared infix (e.g., if has more than two
--- fields), but we will fall back on NormalC in such a scenario.
-conToTH (DCon [] [] n (DNormalC _ stys) _) =
-  NormalC n (map (bangToStrict *** typeToTH) stys)
-conToTH (DCon [] [] n (DRecC vstys) _) =
-  RecC n (map (\(v,b,t) -> (v,bangToStrict b,typeToTH t)) vstys)
-#endif
-#if __GLASGOW_HASKELL__ > 710
 -- On GHC 8.0 or later, we sweeten every constructor to GADT syntax, so it is
 -- perfectly OK to put all of the quantified type variables
 -- (both universal and existential) in a ForallC.
 conToTH (DCon tvbs cxt n fields rty) =
   ForallC (map tvbToTH tvbs) (cxtToTH cxt) (conToTH $ DCon [] [] n fields rty)
-#else
--- On GHCs earlier than 8.0, we must be careful, since the only time ForallC is
--- used is when there are either:
---
--- 1. Any existentially quantified type variables
--- 2. A constructor context
---
--- If neither of these conditions hold, then we needn't put a ForallC at the
--- front, since it would be completely pointless (you'd end up with things like
--- @data Foo = forall. MkFoo@!).
-conToTH (DCon tvbs cxt n fields rty)
-  | null ex_tvbs && null cxt
-  = con'
-  | otherwise
-  = ForallC ex_tvbs (cxtToTH cxt) con'
-  where
-    -- Fortunately, on old GHCs, it's especially easy to distinguish between
-    -- universally and existentially quantified type variables. When desugaring
-    -- a ForallC, we just stick all of the universals (from the datatype
-    -- definition) at the front of the @forall@. Therefore, it suffices to
-    -- count the number of type variables in the return type and drop that many
-    -- variables from the @forall@ in the ForallC, leaving only the
-    -- existentials.
-    ex_tvbs :: [TyVarBndr]
-    ex_tvbs = map tvbToTH $ drop num_univ_tvs tvbs
-
-    num_univ_tvs :: Int
-    num_univ_tvs = go rty
-      where
-        go :: DType -> Int
-        go (DAppT t1 t2) = go t1 + go t2
-        go (DSigT t _)   = go t
-        go (DVarT {})    = 1
-        go (DConT {})    = 0
-        go DArrowT       = 0
-        go (DLitT {})    = 0
-        -- These won't show up on pre-8.0 GHCs
-        go (DForallT {})      = error "`forall` type used in GADT return type"
-        go (DConstrainedT {}) = error "Constrained type used in GADT return type"
-        go DWildCardT         = 0
-        go (DAppKindT {})     = 0
-
-    con' :: Con
-    con' = conToTH $ DCon [] [] n fields rty
-#endif
 
 instanceDToTH :: Maybe Overlap -> DCxt -> DType -> [DDec] -> Dec
-instanceDToTH _over cxt ty decs =
-  InstanceD
-#if __GLASGOW_HASKELL__ >= 800
-            _over
-#endif
-            (cxtToTH cxt) (typeToTH ty) (decsToTH decs)
+instanceDToTH over cxt ty decs =
+  InstanceD over (cxtToTH cxt) (typeToTH ty) (decsToTH decs)
 
 standaloneDerivDToTH :: Maybe DDerivStrategy -> DCxt -> DType -> Dec
-#if __GLASGOW_HASKELL__ >= 710
 standaloneDerivDToTH _mds cxt ty =
   StandaloneDerivD
 #if __GLASGOW_HASKELL__ >= 802
                    (fmap derivStrategyToTH _mds)
 #endif
                    (cxtToTH cxt) (typeToTH ty)
-#else
-standaloneDerivDToTH _ _ _ = error "Standalone deriving supported only in GHC 7.10+"
-#endif
 
 foreignToTH :: DForeign -> Foreign
 foreignToTH (DImportF cc safety str n ty) =
@@ -374,16 +244,17 @@ pragmaToTH (DRuleP str _ rbs lhs rhs phases) =
   RuleP str (map ruleBndrToTH rbs) (expToTH lhs) (expToTH rhs) phases
 #endif
 pragmaToTH (DAnnP target exp) = AnnP target (expToTH exp)
-#if __GLASGOW_HASKELL__ < 709
-pragmaToTH (DLineP {}) = error "LINE pragmas only supported in GHC 7.10+"
-#else
 pragmaToTH (DLineP n str) = LineP n str
-#endif
 #if __GLASGOW_HASKELL__ < 801
 pragmaToTH (DCompleteP {}) = error "COMPLETE pragmas only supported in GHC 8.2+"
 #else
 pragmaToTH (DCompleteP cls mty) = CompleteP cls mty
 #endif
+#if __GLASGOW_HASKELL__ >= 903
+pragmaToTH (DOpaqueP n) = OpaqueP n
+#else
+pragmaToTH (DOpaqueP {}) = error "OPAQUE pragmas only supported in GHC 9.4+"
+#endif
 
 ruleBndrToTH :: DRuleBndr -> RuleBndr
 ruleBndrToTH (DRuleVar n) = RuleVar n
@@ -434,11 +305,7 @@ typeToTH (DVarT n)              = VarT n
 typeToTH (DConT n)              = tyconToTH n
 typeToTH DArrowT                = ArrowT
 typeToTH (DLitT lit)            = LitT lit
-#if __GLASGOW_HASKELL__ > 710
 typeToTH DWildCardT = WildCardT
-#else
-typeToTH DWildCardT = error "Wildcards supported only in GHC 8.0+"
-#endif
 #if __GLASGOW_HASKELL__ >= 807
 typeToTH (DAppKindT t k)        = AppKindT (typeToTH t) (typeToTH k)
 #else
@@ -483,44 +350,13 @@ patSynDirToTH (DExplBidir clauses) = ExplBidir (map clauseToTH clauses)
 #endif
 
 predToTH :: DPred -> Pred
-#if __GLASGOW_HASKELL__ < 709
-predToTH = go []
-  where
-    go acc (DAppT p t) = go (typeToTH t : acc) p
-    -- In the event that we're on a version of Template Haskell without support
-    -- for kind applications, we will simply drop the applied kind.
-    go acc (DAppKindT t _) = go acc t
-    go acc (DSigT p _) = go acc                p  -- this shouldn't happen.
-    go acc (DConT n)
-      | nameBase n == "~"
-      , [t1, t2] <- acc
-      = EqualP t1 t2
-      | otherwise
-      = ClassP n acc
-    go _   (DVarT _)
-      = error "Template Haskell in GHC <= 7.8 does not support variable constraints."
-    go _ DWildCardT
-      = error "Wildcards supported only in GHC 8.0+"
-    go _ (DForallT {})
-      = error "Quantified constraints supported only in GHC 8.6+"
-    go _ (DConstrainedT {})
-      = error "Quantified constraints supported only in GHC 8.6+"
-    go _ DArrowT
-      = error "(->) spotted at head of a constraint"
-    go _ (DLitT {})
-      = error "Type-level literal spotted at head of a constraint"
-#else
 predToTH (DAppT p t) = AppT (predToTH p) (typeToTH t)
 predToTH (DSigT p k) = SigT (predToTH p) (typeToTH k)
 predToTH (DVarT n)   = VarT n
 predToTH (DConT n)   = typeToTH (DConT n)
 predToTH DArrowT     = ArrowT
 predToTH (DLitT lit) = LitT lit
-#if __GLASGOW_HASKELL__ > 710
 predToTH DWildCardT  = WildCardT
-#else
-predToTH DWildCardT  = error "Wildcards supported only in GHC 8.0+"
-#endif
 #if __GLASGOW_HASKELL__ >= 805
 -- We need a special case for DForallT ForallInvis followed by DConstrainedT
 -- so that we may collapse them into a single ForallT when sweetening.
@@ -543,15 +379,12 @@ predToTH (DAppKindT p k) = AppKindT (predToTH p) (typeToTH k)
 -- kind applications, we will simply drop the applied kind.
 predToTH (DAppKindT p _) = predToTH p
 #endif
-#endif
 
 tyconToTH :: Name -> Type
 tyconToTH n
   | n == ''(->)                 = ArrowT -- Work around Trac #14888
   | n == ''[]                   = ListT
-#if __GLASGOW_HASKELL__ >= 709
   | n == ''(~)                  = EqualityT
-#endif
   | n == '[]                    = PromotedNilT
   | n == '(:)                   = PromotedConsT
   | Just deg <- tupleNameDegree_maybe n
@@ -571,11 +404,3 @@ tyconToTH n
 typeArgToTH :: DTypeArg -> TypeArg
 typeArgToTH (DTANormal t) = TANormal (typeToTH t)
 typeArgToTH (DTyArg k)    = TyArg    (typeToTH k)
-
-#if __GLASGOW_HASKELL__ <= 710
--- | Convert a 'Bang' to a 'Strict'
-bangToStrict :: Bang -> Strict
-bangToStrict (Bang SourceUnpack _) = Unpacked
-bangToStrict (Bang _ SourceStrict) = IsStrict
-bangToStrict (Bang _ _)            = NotStrict
-#endif
