@@ -10,11 +10,8 @@ rae@cs.brynmawr.edu
              DataKinds, PolyKinds, GADTs, MultiParamTypeClasses,
              FunctionalDependencies, FlexibleInstances, StandaloneDeriving,
              DefaultSignatures, ConstraintKinds, GADTs, ViewPatterns,
-             TupleSections, NoMonomorphismRestriction #-}
-
-#if __GLASGOW_HASKELL__ >= 711
-{-# LANGUAGE TypeApplications #-}
-#endif
+             TupleSections, NoMonomorphismRestriction, TypeOperators,
+             TypeApplications #-}
 
 #if __GLASGOW_HASKELL__ >= 801
 {-# LANGUAGE DerivingStrategies #-}
@@ -44,13 +41,18 @@ rae@cs.brynmawr.edu
 {-# LANGUAGE QualifiedDo #-}
 #endif
 
-{-# OPTIONS_GHC -fno-warn-missing-signatures -fno-warn-type-defaults
-                -fno-warn-name-shadowing #-}
+#if __GLASGOW_HASKELL__ >= 902
+{-# LANGUAGE OverloadedRecordDot #-}
+#endif
+
+{-# OPTIONS_GHC -Wno-missing-signatures -Wno-type-defaults
+                -Wno-name-shadowing #-}
 
 module Splices where
 
-import Data.List
+import qualified Data.List as L
 import Data.Char
+import qualified Data.Kind as Kind (Type)
 import GHC.Exts
 import GHC.TypeLits
 
@@ -80,21 +82,14 @@ testDecSplice n = do
   infoReg <- reify regName
   rolesDs  <- reifyRoles dsName
   rolesReg <- reifyRoles regName
-#if __GLASGOW_HASKELL__ < 711
-  eqTHSplice (infoDs, rolesDs) (infoReg, rolesReg)
-#else
   fixityDs  <- reifyFixity dsName
   fixityReg <- reifyFixity regName
   eqTHSplice (infoDs, rolesDs, fixityDs) (infoReg, rolesReg, fixityReg)
-#endif
 
 unqualify :: Data a => a -> a
 unqualify = everywhere (mkT (mkName . nameBase))
 
 assumeStarT :: Data a => a -> a
-#if __GLASGOW_HASKELL__ < 709
-assumeStarT = id
-#else
 assumeStarT = everywhere (mkT assume_spec . mkT assume_unit)
   where
     assume_spec :: TyVarBndrSpec -> TyVarBndrSpec
@@ -108,7 +103,6 @@ assumeStarT = everywhere (mkT assume_spec . mkT assume_unit)
     assume_unit :: TyVarBndrUnit -> TyVarBndrUnit
     assume_unit = elimTV (\n   -> kindedTV n StarT)
                          (\n k -> kindedTV n (assumeStarT k))
-#endif
 
 dropTrailing0s :: Data a => a -> a
 dropTrailing0s = everywhere (mkT (mkName . frob . nameBase))
@@ -116,7 +110,7 @@ dropTrailing0s = everywhere (mkT (mkName . frob . nameBase))
     frob str
       | head str == 'r' = str
       | head str == 'R' = str
-      | otherwise       = dropWhileEnd isDigit str
+      | otherwise       = L.dropWhileEnd isDigit str
 
 -- Because th-desugar does not support linear types, we must pretend like
 -- MulArrowT does not exist for testing purposes.
@@ -149,8 +143,8 @@ test7_let = [| let { x :: Double; x = 5; f :: Double -> Double; f x = x + 1 } in
 test8_case = [| case Just False of { Just True -> 1 ; Just _ -> 2 ; Nothing -> 3 } |]
 test9_do = [| show $ do { foo <- Just "foo"
                         ; let fool = foo ++ "l"
-                        ; elemIndex 'o' fool
-                        ; x <- elemIndex 'l' fool
+                        ; L.elemIndex 'o' fool
+                        ; x <- L.elemIndex 'l' fool
                         ; return (x + 10) } |]
 test10_comp = [| [ (x, x+1) | x <- [1..10], x `mod` 2 == 0 ] |]
 test11_parcomp = [| [ (x,y) | x <- [1..10], x `mod` 2 == 0 | y <- [2,5..20] ] |]
@@ -209,13 +203,13 @@ test29_listt = [| let f :: [[a]] -> a
 test30_promoted = [| let f :: Proxy '() -> Proxy '[Int, Bool] -> ()
                          f _ _ = () in
                      f Proxy Proxy |]
-test31_constraint = [| let f :: Proxy (c :: * -> Constraint) -> ()
+test31_constraint = [| let f :: Proxy (c :: Kind.Type -> Constraint) -> ()
                            f _ = () in
                        [f (Proxy :: Proxy Eq), f (Proxy :: Proxy Show)] |]
 test32_tylit = [| let f :: Proxy (a :: Symbol) -> Proxy (b :: Nat) -> ()
                       f _ _ = () in
                   f (Proxy :: Proxy "Hi there!") (Proxy :: Proxy 10) |]
-test33_tvbs = [| let f :: forall a (b :: * -> *). Monad b => a -> b a
+test33_tvbs = [| let f :: forall a (b :: Kind.Type -> Kind.Type). Monad b => a -> b a
                      f = return in
                  [f 1, f 2] :: [Maybe Int] |]
 
@@ -232,11 +226,9 @@ test36_expand = [| let f :: Constant Int (,) Bool Char -> Char
                        f = snd in
                    f |]
 
-#if __GLASGOW_HASKELL__ >= 711
 test40_wildcards = [| let f :: (Show a, _) => a -> a -> _
                           f x y = if x == y then show x else "bad" in
                       f True False :: String |]
-#endif
 
 #if __GLASGOW_HASKELL__ >= 801
 test41_typeapps = [| let f :: forall a. (a -> Bool) -> Bool
@@ -325,6 +317,40 @@ test52_qual_do =
           P.return y |]
 #endif
 
+#if __GLASGOW_HASKELL__ >= 901
+test53_vta_in_con_pats =
+  [| let f :: Maybe Int -> Int
+         f (Just @Int x)  = x
+         f (Nothing @Int) = 42
+     in f (Just @Int 27) |]
+#endif
+
+#if __GLASGOW_HASKELL__ >= 902
+data ORD1 = MkORD1 { unORD1 :: Int }
+data ORD2 = MkORD2 { unORD2 :: ORD1 }
+
+test54_overloaded_record_dot =
+  [| let ord1 :: ORD1
+         ord1 = MkORD1 1
+
+         ord2 :: ORD2
+         ord2 = MkORD2 ord1
+
+     in (ord2.unORD2.unORD1, (.unORD2.unORD1) ord2) |]
+#endif
+
+#if __GLASGOW_HASKELL__ >= 903
+test55_opaque_pragma =
+  [| let f :: String -> String
+         f x = x
+         {-# OPAQUE f #-}
+     in f "Hello, World!" |]
+
+test56_lambda_cases =
+  [| (\cases (Just x) (Just y) -> x ++ y
+             _        _        -> "") (Just "Hello") (Just "World") |]
+#endif
+
 type family TFExpand x
 type instance TFExpand Int = Bool
 type instance TFExpand (Maybe a) = [a]
@@ -347,10 +373,10 @@ test_expand6 = [| let f :: ClosedTF Double -> ()
                   f |]
 
 #if __GLASGOW_HASKELL__ >= 809
-type PolyTF :: forall k. k -> *
+type PolyTF :: forall k. k -> Kind.Type
 #endif
-type family PolyTF (x :: k) :: * where
-  PolyTF (x :: *) = Bool
+type family PolyTF (x :: k) :: Kind.Type where
+  PolyTF (x :: Kind.Type) = Bool
 
 test_expand7 = [| let f :: PolyTF Int -> ()
                       f True = () in
@@ -360,13 +386,17 @@ test_expand8 = [| let f :: PolyTF IO -> ()
                   f |]
 
 
-#if __GLASGOW_HASKELL__ >= 709
 test_expand9 = [| let f :: TFExpand (Maybe (IO a)) -> IO ()
                       f actions = sequence_ actions in
                   f |]
-#endif
 
-#if __GLASGOW_HASKELL__ >= 709
+type family TFExpandClosed a where
+  TFExpandClosed (Maybe a) = [a]
+
+test_expand10 = [| let f :: TFExpandClosed (Maybe (IO a)) -> IO ()
+                       f actions = sequence_ actions in
+                   f |]
+
 test37_pred = [| let f :: (Read a, (Show a, Num a)) => a -> a
                      f x = read (show x) + x in
                  (f 3, f 4.5) |]
@@ -378,13 +408,8 @@ test38_pred2 = [| let f :: a b => Proxy a -> b -> b
 test39_eq = [| let f :: (a ~ b) => a -> b
                    f x = x in
                (f ()) |]
-#endif
 
-#if __GLASGOW_HASKELL__ < 709
-dec_test_nums = [1..10] :: [Int]
-#else
 dec_test_nums = [1..11] :: [Int]
-#endif
 
 dectest1 = [d| data Dec1 where
                  Foo :: Dec1
@@ -398,25 +423,23 @@ dectest3 = [d| data Dec3 a where
 dectest4 = [d| newtype Dec4 a where
                  MkDec4 :: (a, Int) -> Dec4 a |]
 dectest5 = [d| type Dec5 a b = (a b, Maybe b) |]
-dectest6 = [d| class (Monad m1, Monad m2) => Dec6 (m1 :: * -> *) m2 | m1 -> m2  where
+dectest6 = [d| class (Monad m1, Monad m2) => Dec6 (m1 :: Kind.Type -> Kind.Type) m2 | m1 -> m2  where
                  lift :: forall a. m1 a -> m2 a
-                 type M2 m1 :: * -> * |]
-dectest7 = [d| type family Dec7 a (b :: *) (c :: Bool) :: * -> * |]
+                 type M2 m1 :: Kind.Type -> Kind.Type |]
+dectest7 = [d| type family Dec7 a (b :: Kind.Type) (c :: Bool) :: Kind.Type -> Kind.Type |]
 dectest8 = [d| type family Dec8 a |]
-dectest9 = [d| data family Dec9 a (b :: * -> *) :: * -> * |]
-dectest10 = [d| type family Dec10 a :: * -> * where
+dectest9 = [d| data family Dec9 a (b :: Kind.Type -> Kind.Type) :: Kind.Type -> Kind.Type |]
+dectest10 = [d| type family Dec10 a :: Kind.Type -> Kind.Type where
                   Dec10 Int = Maybe
                   Dec10 Bool = [] |]
 
 data Blarggie a = MkBlarggie Int a
-#if __GLASGOW_HASKELL__ >= 709
 dectest11 = [d| class Dec11 a where
                   meth13 :: a -> a -> Bool
                   default meth13 :: Eq a => a -> a -> Bool
                   meth13 = (==)
               |]
 standalone_deriving_test = [d| deriving instance Eq a => Eq (Blarggie a) |]
-#endif
 #if __GLASGOW_HASKELL__ >= 801
 deriv_strat_test = [d| deriving stock instance Ord a => Ord (Blarggie a) |]
 #endif
@@ -427,7 +450,7 @@ dectest12 = [d| data Dec12 a where
 
               |]
 
-dectest13 = [d| data Dec13 :: (* -> Constraint) -> * where
+dectest13 = [d| data Dec13 :: (Kind.Type -> Constraint) -> Kind.Type where
                   MkDec13 :: c a => a -> Dec13 c
               |]
 
@@ -445,33 +468,35 @@ class ExCls a
 data ExData1 a
 data ExData2 a
 
+-- ds_dectest{16,17} demonstrate instance declarations with outermost foralls,
+-- a feature which Template Haskell itself does not yet support (see #151).
+-- For this reason, the closest we can get to this in TH is to construct
+-- equivalent Decs, dectest{16,17}, that drop the outermost foralls. The test
+-- suite ensures that this process happens automatically during sweetening by
+-- checking that the sweetened versions of ds_dectest{16,17} equal
+-- dectest{16,17}.
+
 ds_dectest16 = DInstanceD Nothing (Just [DPlainTV (mkName "a") ()]) []
                 (DConT ''ExCls `DAppT`
                   (DConT ''ExData1 `DAppT` DVarT (mkName "a"))) []
 dectest16 :: Q [Dec]
 dectest16 = return [ InstanceD
-#if __GLASGOW_HASKELL__ >= 800
                        Nothing
-#endif
-                       [] (ForallT [plainTVSpecified (mkName "a")] []
-                                   (ConT ''ExCls `AppT`
-                                     (ConT ''ExData1 `AppT` VarT (mkName "a")))) [] ]
+                       [] (ConT ''ExCls `AppT`
+                            (ConT ''ExData1 `AppT` VarT (mkName "a"))) [] ]
 ds_dectest17 = DStandaloneDerivD Nothing (Just [DPlainTV (mkName "a") ()]) []
                 (DConT ''ExCls `DAppT`
                   (DConT ''ExData2 `DAppT` DVarT (mkName "a")))
-#if __GLASGOW_HASKELL__ >= 710
 dectest17 :: Q [Dec]
 dectest17 = return [ StandaloneDerivD
 #if __GLASGOW_HASKELL__ >= 802
                        Nothing
 #endif
-                       [] (ForallT [plainTVSpecified (mkName "a")] []
-                                   (ConT ''ExCls `AppT`
-                                     (ConT ''ExData2 `AppT` VarT (mkName "a")))) ]
-#endif
+                       [] (ConT ''ExCls `AppT`
+                            (ConT ''ExData2 `AppT` VarT (mkName "a"))) ]
 
 #if __GLASGOW_HASKELL__ >= 809
-dectest18 = [d| data Dec18 :: forall k -> k -> * where
+dectest18 = [d| data Dec18 :: forall k -> k -> Kind.Type where
                   MkDec18 :: forall k (a :: k). Dec18 k a |]
 #endif
 
@@ -484,26 +509,18 @@ imp_inst_test1 = [d| instance Dec6 Maybe (Either ()) where
                        lift (Just x) = Right x
                        type M2 Maybe = Either () |]
 
-data family Dec9 a (b :: * -> *) :: * -> *
-#if __GLASGOW_HASKELL__ >= 800
+data family Dec9 a (b :: Kind.Type -> Kind.Type) :: Kind.Type -> Kind.Type
 imp_inst_test2 = [d| data instance Dec9 Int Maybe a where
                        MkIMB  ::             [a] -> Dec9 Int Maybe a
                        MkIMB2 :: forall a b. b a -> Dec9 Int Maybe a |]
 imp_inst_test3 = [d| newtype instance Dec9 Bool m x where
                        MkBMX :: m x -> Dec9 Bool m x |]
-#else
--- TH-quoted data family instances with GADT syntax are horribly broken on GHC 7.10
--- and older, so we opt to use non-GADT syntax on older GHCs so we can at least
--- test *something*.
-imp_inst_test2 = [d| data instance Dec9 Int Maybe a = MkIMB [a] | forall b. MkIMB2 (b a) |]
-imp_inst_test3 = [d| newtype instance Dec9 Bool m x = MkBMX (m x) |]
-#endif
 
 type family Dec8 a
 imp_inst_test4 = [d| type instance Dec8 Int = Bool |]
 
 -- used for bug8884 test
-type family Poly (a :: k) :: *
+type family Poly (a :: k) :: Kind.Type
 type instance Poly x = Int
 
 flatten_dvald_test = [| let (a,b,c) = ("foo", 4, False) in
@@ -519,13 +536,8 @@ rec_sel_test_num_sels = 4 :: Int
 
 testRecSelTypes :: Int -> Q Exp
 testRecSelTypes n = do
-#if __GLASGOW_HASKELL__ > 710
   VarI _ ty1 _ <- reify (mkName ("DsDec.recsel" ++ show n))
   VarI _ ty2 _ <- reify (mkName ("Dec.recsel"   ++ show n))
-#else
-  VarI _ ty1 _ _ <- reify (mkName ("DsDec.recsel" ++ show n))
-  VarI _ ty2 _ _ <- reify (mkName ("Dec.recsel"   ++ show n))
-#endif
   let ty1' = return $ unqualify ty1
       ty2' = return $ unqualify ty2
   [| let x :: $ty1'
@@ -550,13 +562,11 @@ reifyDecs = [d|
 
   class R2 a b where
     r3 :: a -> b -> c -> a
-    type R4 b a :: *
-#if __GLASGOW_HASKELL__ >= 800
+    type R4 b a :: Kind.Type
     -- Only define this on GHC 8.0 or later, since TH had trouble quoting
     -- associated type family defaults before then.
     type R4 b a = Either a b
-#endif
-    data R5 a :: *
+    data R5 a :: Kind.Type
 
   data R6 a = R7 { r8 :: a -> a, r9 :: Bool }
 
@@ -565,9 +575,9 @@ reifyDecs = [d|
     type R4 a (R6 a) = a
     data R5 (R6 a) = forall b. Show b => R10 { r11 :: a, naughty :: b }
 
-  type family R12 a b :: *
+  type family R12 a b :: Kind.Type
 
-  data family R13 a :: *
+  data family R13 a :: Kind.Type
 
   data instance R13 Int = R14 { r15 :: Bool }
 
@@ -637,19 +647,17 @@ reifyDecs = [d|
     deriving Eq via (Id [a])
 #endif
 
-#if __GLASGOW_HASKELL__ >= 800
-  class R25 (f :: k -> *) where
+  class R25 (f :: k -> Kind.Type) where
     r26 :: forall (a :: k). f a
 
   data R27 (a :: k) = R28 { r29 :: Proxy a }
-#endif
 
   class R30 a where
     r31 :: a -> b -> a
 
 #if __GLASGOW_HASKELL__ >= 809
-  type R32 :: forall k -> k -> *
-  type family R32 :: forall k -> k -> * where
+  type R32 :: forall k -> k -> Kind.Type
+  type family R32 :: forall k -> k -> Kind.Type where
 #endif
 
   data R33 a where
@@ -659,16 +667,11 @@ reifyDecs = [d|
 reifyDecsNames :: [Name]
 reifyDecsNames = map mkName
   [ "r1"
-#if __GLASGOW_HASKELL__ < 711
-  , "R2", "r3"      -- these fail due to GHC#11797
-#endif
   , "R4", "R5", "R6", "R7", "r8", "r9", "R10", "r11"
   , "R12", "R13", "R14", "r15", "r16", "r17", "R18", "R19", "R20"
   , "R21"
   , "r22"
-#if __GLASGOW_HASKELL__ >= 800
   , "R25", "r26", "R28", "r29"
-#endif
   , "R30", "r31"
 #if __GLASGOW_HASKELL__ >= 809
   , "R32"
@@ -735,11 +738,9 @@ test_exprs = [ test1_sections
              , test32_tylit
              , test33_tvbs
              , test34_let_as
-#if __GLASGOW_HASKELL__ >= 709
              , test37_pred
              , test38_pred2
              , test39_eq
-#endif
 #if __GLASGOW_HASKELL__ >= 801
              , test41_typeapps
              , test42_scoped_tvs
@@ -763,5 +764,15 @@ test_exprs = [ test1_sections
 #endif
 #if __GLASGOW_HASKELL__ >= 900
              , test52_qual_do
+#endif
+#if __GLASGOW_HASKELL__ >= 901
+             , test53_vta_in_con_pats
+#endif
+#if __GLASGOW_HASKELL__ >= 902
+             , test54_overloaded_record_dot
+#endif
+#if __GLASGOW_HASKELL__ >= 903
+             , test55_opaque_pragma
+             , test56_lambda_cases
 #endif
              ]
